/* 
 * This file is part of the Track+ application, a 
 * tool for project and change management. 
 * 
 * Copyright (C) 2009 Trackplus 
 * 
 * Use and distribution of this software is governed by the 
 * Track+ license conditions. 
 * 
 * $Id: AccountingDatasourceBase.java 2688 2010-11-04 16:47:10Z friedj $ 
 * 
 */

package com.trackplus.plugin.report.accounting;		
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import com.aurel.track.beans.TPersonBean;
import com.aurel.track.beans.TProjectBean;
import com.aurel.track.dao.CostDAO;
import com.aurel.track.dao.DAOFactory;
import com.aurel.track.plugin.DatasourceDescriptor;
import com.aurel.track.report.datasource.DatasourceCommonBL;
import com.aurel.track.report.datasource.IPluggableDatasource;
import com.aurel.track.util.ArrayConverter;
import com.aurel.track.util.CalendarUtil;
import com.aurel.track.util.DateTimeUtils;
import com.aurel.track.util.GeneralUtils;
import com.aurel.track.util.StringArrayParameterUtils;

public abstract class AccountingDatasourceBase implements IPluggableDatasource {
		
	private static CostDAO costDAO = DAOFactory.getFactory().getCostDAO();
	/** 
	 * The defined operation modes
	 * 0 - overview of own costs
	 * 1 - overview of all costs
	 */
	protected static class OperationMode {
		static public final int PERSONALOVERVIEW = 0;
		static public final int MANAGEROVERVIEW = 1;		
	}
	
	/** 
	 * The defined time intervals 	 
	 */
	protected static class TimeInterval {
		static public final int CURRENTMONTH = 0;
		static public final int LASTMONTH = 1;
		static public final int FROMTO = 2;
	}
		
	/** 
	 * The defined operation modes
	 * 0 - overview of own costs
	 * 1 - overview of all costs
	 */
	protected static class AccountigReports {
		static public final int PERSON_PROJECT_ACCOUNT_SUM = 0;
		static public final int PROJECT_COSTCENTER_ACCOUNT_PERSON_SUM = 1;		
		static public final int PROJECT_DATE_ACCOUNT_PERSON_DETAIL = 2;
	}
	
	protected static interface PARAMETER_NAME {
		public static String locale = "locale";
		public static String bundleName = "bundleName";
		public static String jsCalDateFormat = "jsCalDateFormat";
		public static String timeIntervals = "timeIntervals";
		public static String selectedTimeInterval = "selectedTimeInterval";
		public static String dateFrom = "dateFrom";
		public static String dateTo = "dateTo";
		public static String operationModes = "operationModes";
		public static String selectedOperationMode = "selectedOperationMode";		
		public static String projects = "projects";
		public static String selectedProjects = "selectedProjects";		
		public static String personDisabled = "personDisabled";
		public static String departments = "departments";
		public static String selectedDepartments = "selectedDepartments";
		public static String persons = "persons";
		public static String selectedPersons = "selectedPersons";
		public static String costcenters = "costcenters";
		public static String selectedCostcenters = "selectedCostcenters";
		public static String accounts = "accounts";
		public static String selectedAccounts = "selectedAccounts";
		public static String fromTo = "fromTo"; 
		public static String action = "action";
	}
	
	/**
	 * Prepares the parameters map:
	 * 1. 	for first rendering
	 * 2. 	after each submit which needs re-rendering of the configuration page 
	 * 		(for example refreshing some lists etc., typically using a  
	 * 		 hidden field set through javascript before submit) 
	 * 3. 	after the submit for getting the datasource or generating the report     
	 */
	public Map prepareParameters(Map parameters, DatasourceDescriptor datasourceDescriptor, 
			TPersonBean personBean, Locale locale) {
		String bundleName = datasourceDescriptor.getBundleName();
		if (parameters == null) {
			//when no submit took place parameters is null 
			parameters = new HashMap();
		}
		parameters.put(PARAMETER_NAME.locale, locale);
		parameters.put(PARAMETER_NAME.bundleName, bundleName);
		parameters.put(PARAMETER_NAME.jsCalDateFormat, DateTimeUtils.getInstance().getJsCalDateFormat(locale));		
		parameters.put(PARAMETER_NAME.timeIntervals, AccountingDatasourceBL.getLocalizedTimeIntervals(locale, bundleName));
		parameters.put(PARAMETER_NAME.operationModes,  AccountingDatasourceBL.getAccountingReportOperationModes(locale, bundleName));
		int selectedOperationMode = StringArrayParameterUtils.parseIntegerValue(parameters, 
				PARAMETER_NAME.selectedOperationMode, OperationMode.PERSONALOVERVIEW);		
		List<TProjectBean> projectBeanList;
		
		switch (selectedOperationMode) {
    	case OperationMode.PERSONALOVERVIEW:
    		projectBeanList = AccountingDatasourceBL.getProjectsOptionsListByUserAsOwn(personBean.getObjectID());
    		parameters.put(PARAMETER_NAME.projects, 
    				DatasourceCommonBL.getProjectsAndReleasesByProjects(projectBeanList));
    		parameters.put(PARAMETER_NAME.personDisabled, new Boolean(true));
			break;
    	case OperationMode.MANAGEROVERVIEW:
    		projectBeanList = AccountingDatasourceBL.getProjectsOptionsListByUserAsManager(personBean.getObjectID());
    		parameters.put(PARAMETER_NAME.projects, 
    				DatasourceCommonBL.getProjectsAndReleasesByProjects(projectBeanList));
    		parameters.put(PARAMETER_NAME.personDisabled, new Boolean(false));	    			    		
    		break;		
    	}		
		parameters.put(PARAMETER_NAME.departments, AccountingDatasourceBL.getAllDepartmentsOptionsList());
		parameters.put(PARAMETER_NAME.persons, AccountingDatasourceBL.getAllPersonsOptionsList());
		parameters.put(PARAMETER_NAME.costcenters, AccountingDatasourceBL.getAllCostcentersOptionsList());
		parameters.put(PARAMETER_NAME.accounts, AccountingDatasourceBL.getAllAccountsOptionsList());  						    	
    	
		int selectedTimeInterval = StringArrayParameterUtils.parseIntegerValue(parameters, 
				PARAMETER_NAME.selectedTimeInterval, TimeInterval.CURRENTMONTH);
		parameters.put(PARAMETER_NAME.selectedTimeInterval, new Integer(selectedTimeInterval));				
		parameters.put(PARAMETER_NAME.fromTo, 
				new Boolean(selectedTimeInterval==TimeInterval.FROMTO));
		
		if (parameters.get(PARAMETER_NAME.selectedOperationMode)==null) {
			parameters.put(PARAMETER_NAME.selectedOperationMode, new Integer(OperationMode.PERSONALOVERVIEW).toString());
		}				
		//for correct rerendering transform the String[]s to Integer[]s
		parameters.put(PARAMETER_NAME.selectedProjects, 
				StringArrayParameterUtils.parseIntegerArrValue(parameters, PARAMETER_NAME.selectedProjects));		
		parameters.put(PARAMETER_NAME.selectedDepartments, 
				StringArrayParameterUtils.parseIntegerArrValue(parameters, PARAMETER_NAME.selectedDepartments));		
		parameters.put(PARAMETER_NAME.selectedPersons, 
				StringArrayParameterUtils.parseIntegerArrValue(parameters, PARAMETER_NAME.selectedPersons));			
		parameters.put(PARAMETER_NAME.selectedCostcenters, 
				StringArrayParameterUtils.parseIntegerArrValue(parameters, PARAMETER_NAME.selectedCostcenters));
		parameters.put(PARAMETER_NAME.selectedAccounts, 
				StringArrayParameterUtils.parseIntegerArrValue(parameters, PARAMETER_NAME.selectedAccounts));
				
		String action = StringArrayParameterUtils.getStringValue(parameters, PARAMETER_NAME.action);
		if (action!=null) {			
			if (action.equals("reloadPersons")) {				
				Integer[] selectedDepartments = (Integer[])parameters.get(PARAMETER_NAME.selectedDepartments);   	    	    	
		    	parameters.put(PARAMETER_NAME.persons, 
		    			AccountingDatasourceBL.getPersonsOptionsListByDepartments(selectedDepartments)); 
			}
			if (action.equals("reloadAccounts")) {
				Integer[] selectedCostcenters = (Integer[])parameters.get(PARAMETER_NAME.selectedCostcenters);				
		    	parameters.put(PARAMETER_NAME.accounts, 
		    			AccountingDatasourceBL.getAccountsOptionsListByCostcenters(selectedCostcenters));		    	
		    	return parameters;
			}
		}				
		return parameters;
	}
	
	/**
	 * Get the names of those parameter names which should be saved in the session
	 * for future use of the same report configuration
	 * @return
	 */
	public List getSavedParameterNames() {
		List savedParameterNames = new ArrayList();
		savedParameterNames.add(PARAMETER_NAME.selectedTimeInterval);
		savedParameterNames.add(PARAMETER_NAME.dateFrom);
		savedParameterNames.add(PARAMETER_NAME.dateTo);		
		savedParameterNames.add(PARAMETER_NAME.selectedOperationMode);
		savedParameterNames.add(PARAMETER_NAME.selectedProjects);
		savedParameterNames.add(PARAMETER_NAME.selectedDepartments);
		savedParameterNames.add(PARAMETER_NAME.selectedPersons);
		savedParameterNames.add(PARAMETER_NAME.selectedCostcenters);
		savedParameterNames.add(PARAMETER_NAME.selectedAccounts);
		return savedParameterNames;
	}
	
	
	public void serializeDatasource(OutputStream outputStream,
			Object datasource) {				
	}



	public boolean implementSerialization() {
		return false;
	}



	/**
	 * Get 	
	 * @param parameters
	 * @param userID
	 * @return
	 */
	protected AccountingReportConfigStruct getConfigData(Map parameters, Integer userID) {
		loadParametersBeforeExecute(parameters, userID);		
		AccountingReportConfigStruct configStruct = new AccountingReportConfigStruct();								
		int selectedTimeInterval = StringArrayParameterUtils.parseIntegerValue(parameters, 
				PARAMETER_NAME.selectedTimeInterval, TimeInterval.CURRENTMONTH);
		//by project should be filtered anyway (because of the rights)
		List<Integer> selectedProjects = GeneralUtils.createIntegerListFromStringArr((String[])parameters.get(PARAMETER_NAME.selectedProjects));
		if (selectedProjects == null || selectedProjects.size() == 0) {
			if (parameters.get(DatasourceCommonBL.PARAMETER_NAME.PREDEFINED_PROJECT_RELEASE)!=null) {
				//predefined (coming from a dashboard)
				Integer selectedProjectReleaseID = StringArrayParameterUtils.parseIntegerValue(parameters, 
						DatasourceCommonBL.PARAMETER_NAME.SELECTED_PROJECT);
				if (selectedProjectReleaseID!=null) {
					selectedProjects = new ArrayList<Integer>();
					selectedProjects.add(selectedProjectReleaseID);		    		
		}
			} else {
				//not predefined (not from a dashboard)
				//selectedProjects = GeneralUtils.createIntegerListFromBeanList((List)parameters.get(PARAMETER_NAME.projects));
			}
		}
		
		configStruct.setSelectedProjects(selectedProjects);		
		List<Integer> selecetedExplicitProjects = new ArrayList<Integer>();
		List<Integer> selecetedImplicitProjects = new ArrayList<Integer>();
		List<Integer> selectedReleases = new ArrayList<Integer>();		
		DatasourceCommonBL.getProjectsAndReleasesIDs(selectedProjects, 
				selecetedExplicitProjects, selecetedImplicitProjects, selectedReleases);
		
		int selectedOperationMode = StringArrayParameterUtils.parseIntegerValue(parameters, 
				PARAMETER_NAME.selectedOperationMode, OperationMode.PERSONALOVERVIEW);
		
		int[] selectedPersons;		
		if (selectedOperationMode == OperationMode.PERSONALOVERVIEW) {
			selectedPersons = new int[] {userID.intValue()};
		}
		else {
			//by departments will not be filtered directly, it affects only the possible persons
			int[] selectedDepartments = ArrayConverter.stringToInt(
					(String[])parameters.get(PARAMETER_NAME.selectedDepartments));
			//by persons should be filtered only if selected, or department(s) is selected
			selectedPersons = ArrayConverter.stringToInt(
					(String[])parameters.get(PARAMETER_NAME.selectedPersons));
			if ((selectedDepartments != null && selectedDepartments.length > 0) 
					&& (selectedPersons == null || selectedPersons.length == 0)) {
				selectedPersons = ArrayConverter.integerToInt(GeneralUtils.getBeanIDs((List)parameters.get(PARAMETER_NAME.persons)));
			}			
		}
		configStruct.setSelectedPersons(selectedPersons);
		
		//by costcenters will not be filtered directly, it affects only the possible accounts
		int[] selectedCostcenters = ArrayConverter.stringToInt(
				(String[])parameters.get(PARAMETER_NAME.selectedCostcenters));
		//by accounts should be filtered only if selected, or costcenter(s) is selected
		int[] selectedAccounts = ArrayConverter.stringToInt(
				(String[])parameters.get(PARAMETER_NAME.selectedAccounts));
		if ((selectedCostcenters != null && selectedCostcenters.length > 0) 
				&& (selectedAccounts == null || selectedAccounts.length == 0)) {
			selectedAccounts = ArrayConverter.integerToInt(GeneralUtils.getBeanIDs((List)parameters.get(PARAMETER_NAME.accounts)));
		}	
		configStruct.setSelectedAccounts(selectedAccounts);	
		
		Calendar calendar = new GregorianCalendar();
		
		Date dateFrom = null;
		Date dateTo = null;
		Date dateTmp; 		

		switch (selectedTimeInterval) {
			case TimeInterval.CURRENTMONTH: 								
				CalendarUtil.clearTime(calendar);
				calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
				dateFrom = calendar.getTime();
				calendar.add(Calendar.MONTH, 1);
				dateTo = calendar.getTime();
				break;
			case TimeInterval.LASTMONTH:
				CalendarUtil.clearTime(calendar);				
				calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMinimum(Calendar.DAY_OF_MONTH));
				dateTo = calendar.getTime();
				calendar.add(Calendar.MONTH, -1);
				dateFrom = calendar.getTime();
				break;
			case TimeInterval.FROMTO:
				String dateFromArr = StringArrayParameterUtils.getStringValue(parameters, PARAMETER_NAME.dateFrom);
				if (dateFromArr!=null) {
					dateTmp = DateTimeUtils.getInstance().parseGUIDate(dateFromArr, 
							(Locale)parameters.get(PARAMETER_NAME.locale));
					if (dateTmp==null) {
						dateTmp = costDAO.getFirstLastExpenseDate(selecetedExplicitProjects, selectedReleases, selectedPersons, selectedAccounts, true);
						if (dateTmp==null) {
							calendar.setTime(new Date());
						} else {
							calendar.setTime(dateTmp);
						}
					} else {
						calendar.setTime(dateTmp);										
					}
					CalendarUtil.clearTime(calendar);
					dateFrom = calendar.getTime();
				}
				String dateToArr = StringArrayParameterUtils.getStringValue(parameters, PARAMETER_NAME.dateTo);
				if (dateToArr!=null) {
					dateTmp = DateTimeUtils.getInstance().parseGUIDate(dateToArr, 
							(Locale)parameters.get(PARAMETER_NAME.locale));
					if (dateTmp==null) {
						dateTmp = costDAO.getFirstLastExpenseDate(selecetedExplicitProjects, selectedReleases, selectedPersons, selectedAccounts, false);
						if (dateTmp==null) {
							calendar.setTime(new Date());
						} else {
							calendar.setTime(dateTmp);
						}																						
					} else {
						calendar.setTime(dateTmp);
					}
					CalendarUtil.clearTime(calendar);
					calendar.add(Calendar.DATE, 1);
					dateTo = calendar.getTime();
				}
				break;
		}				
		configStruct.setDateFrom(dateFrom);
		configStruct.setDateTo(dateTo);
			
		
			
		return configStruct;
	}
	

	/**
	 * Prepares the lists before execute because they might be needed by filtering the data 
	 * @param parameters
	 * @param personID
	 * @return
	 */
	protected Map loadParametersBeforeExecute(Map parameters, Integer personID) {
		int selectedOperationMode = StringArrayParameterUtils.parseIntegerValue(parameters, 
				PARAMETER_NAME.selectedOperationMode, OperationMode.PERSONALOVERVIEW);
		switch (selectedOperationMode) {
    	case OperationMode.PERSONALOVERVIEW:
    		parameters.put(PARAMETER_NAME.projects, AccountingDatasourceBL.getProjectsOptionsListByUserAsOwn(personID));    		
			break;
    	case OperationMode.MANAGEROVERVIEW:
    		parameters.put(PARAMETER_NAME.projects, AccountingDatasourceBL.getProjectsOptionsListByUserAsManager(personID));    		 			    		
    		break;		
    	}		
		parameters.put(PARAMETER_NAME.departments, AccountingDatasourceBL.getAllDepartmentsOptionsList());
		parameters.put(PARAMETER_NAME.persons, AccountingDatasourceBL.getAllPersonsOptionsList());
		parameters.put(PARAMETER_NAME.costcenters, AccountingDatasourceBL.getAllCostcentersOptionsList());
		parameters.put(PARAMETER_NAME.accounts, AccountingDatasourceBL.getAllAccountsOptionsList());
		return parameters;
	}		
}
